<?php

namespace Drupal\lms_answer_plugins\Plugin\ActivityAnswer;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\lms\Attribute\ActivityAnswer;
use Drupal\lms\Entity\Answer;
use Drupal\lms\Plugin\ActivityAnswer\ActivityAnswerBase;

/**
 * Long Answer activity plugin.
 */
#[ActivityAnswer(
  id: 'free_text',
  name: new TranslatableMarkup('Free text'),
)]
class FreeText extends ActivityAnswerBase {

  /**
   * {@inheritdoc}
   */
  public function evaluatedOnSave(Answer $activity) {
    // Manually evaluated activity.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function answeringForm(array &$form, Answer $answer) {
    $activity = $answer->getActivity();
    $data = $answer->getData();
    $form['answer'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Your answer'),
      '#default_value' => $data['answer'] ?? '',
    ];

    // Include data from the activity entity.
    $description_field = $activity->get('description');
    if (!$description_field->isEmpty()) {
      $form['answer']['#description'] = $description_field->value;
    }
  }

}
