<?php

namespace Drupal\lms\Entity\Handlers;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Defines a class to build a listing of Lesson entities.
 */
class LessonListBuilder extends EntityListBuilder {

  /**
   * Returns lessons count.
   *
   * @return int
   *   The total amount of lesson entities.
   */
  protected function getTotalCount(): int {
    return $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->count()
      ->execute();
  }

  /**
   * Returns the entity IDs of lesson entities.
   *
   * @return string[]|null
   *   An array with entity IDs.
   */
  protected function getEntityIds(): ?array {
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE);
    if ($this->limit) {
      $page = \Drupal::request()->query->get('page', 0);
      $limit = $this->limit;
      $start = $limit * $page;
      $query->range($start, $limit);
    }
    $query->sort($this->entityType->getKey('id'));
    $ids = $query->execute();

    return $ids;
  }

  /**
   * Renders the lesson entities.
   *
   * @return mixed[]
   *   A render array.
   */
  public function render(): array {
    $pager_manager = \Drupal::service('pager.manager');
    $pager_manager->createPager($this->getTotalCount(), $this->limit);

    $build = parent::render();
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader(): array {
    $header['id'] = $this->t('Lesson ID');
    $header['name'] = $this->t('Name');
    $header['owner'] = $this->t('Owner');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity): array {
    $row['id'] = $entity->id();
    $row['name'] = Link::createFromRoute($entity->label(), 'entity.lms_lesson.edit_form', ['lms_lesson' => $entity->id()]);
    $row['owner'] = $entity->get('uid')->entity->toLink();

    // Change default edit link.
    $ops = parent::buildRow($entity);
    $ops['operations']['data']['#links']['edit']['url'] = new Url(
      'entity.lms_lesson.edit_form', [
        'lms_lesson' => $entity->id(),
      ],
      [
        'query' => ['destination' => \Drupal::request()->getRequestUri()],
      ],
    );

    return $row + $ops;
  }

}
