<?php

namespace Drupal\lms\Entity;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\user\EntityOwnerInterface;

/**
 * Provides an interface for defining Lesson entities.
 */
interface LessonInterface extends ContentEntityInterface, EntityChangedInterface, EntityOwnerInterface {

  /**
   * Gets the Lesson creation timestamp.
   *
   * @return int
   *   Creation timestamp of the Lesson.
   */
  public function getCreatedTime();

  /**
   * Sets the Lesson creation timestamp.
   *
   * @param int $timestamp
   *   The Lesson creation timestamp.
   *
   * @return \Drupal\lms\Entity\LessonInterface
   *   The called Lesson entity.
   */
  public function setCreatedTime($timestamp);

  /**
   * Returns the Lesson entity randomization setting.
   *
   * @return string
   *   The Lesson entity randomization setting.
   */
  public function getRandomization(): string;

  /**
   * Returns the Lesson entities' backwards navigation setting.
   *
   * @return int
   *   The entities' backwards navigation setting.
   */
  public function getBackwardsNavigation(): int;

  /**
   * Returns the Lesson entities' random activities count.
   *
   * @return int
   *   The Lesson entities' random activities count.
   */
  public function getRandomActivitiesCount(): int;

  /**
   * Sets the Lesson entities' random activity count.
   *
   * @param int $activityCount
   *   The amount of random activities.
   */
  public function setRandomActivitiesCount(int $activityCount): self;

}
