<?php

declare(strict_types=1);

namespace Drupal\lms\Hooks;

use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\lms\Entity\Bundle\Course;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Handles custom LMS cache tags invalidation.
 */
final class LmsTagsInvalidator implements ContainerInjectionInterface {

  /**
   * The constructor.
   */
  public function __construct(
    protected readonly CacheTagsInvalidatorInterface $cacheTagsInvalidator
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('cache_tags.invalidator')
    );
  }

  /**
   * Invalidate group_relationship related tags.
   */
  public function invalidateRelationshipTags(GroupRelationshipInterface $group_relationship) {
    // Invalidate course classes tag.
    if ($group_relationship->getPluginId() === 'lms_classes') {
      $parent_group = $group_relationship->getGroup();
      if (!$parent_group instanceof Course) {
        return;
      }
      $this->cacheTagsInvalidator->invalidateTags(['group.classes:' . $parent_group->id()]);
    }

    // Invalidate course membership request tag.
    elseif ($group_relationship->getPluginId() === 'group_membership_request') {
      $this->cacheTagsInvalidator->invalidateTags(['lms_course_membership:' . $group_relationship->getGroupId() . ':' . $group_relationship->getEntityId()]);
    }

    // Invalidate course take access tags.
    if ($group_relationship->getPluginId() === 'group_membership') {
      if (\in_array($group_relationship->getGroupTypeId(), [
        'lms_class',
        'lms_course',
      ])) {
        // @todo Make this Course-specific (if a class, get parent group ID).
        $this->cacheTagsInvalidator->invalidateTags(['lms_memberships:' . $group_relationship->getEntityId()]);
      }
    }
  }

}
