<?php

namespace Drupal\lms\Plugin\ActivityAnswer;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\lms\Config\PluginConfigInstaller;
use Drupal\lms\Entity\ActivityType;
use Drupal\lms\Entity\Answer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Activity-answer plugin base class.
 */
abstract class ActivityAnswerBase extends PluginBase implements ActivityAnswerInterface, ContainerFactoryPluginInterface {

  /**
   * The constructor.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected readonly PluginConfigInstaller $pluginConfigInstaller,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.config_installer.activity_answer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getId() {
    return $this->pluginDefinition['id'];
  }

  /**
   * {@inheritdoc}
   */
  public function evaluatedOnSave(Answer $answer) {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getScore(Answer $answer) {
    return 1;
  }

  /**
   * {@inheritdoc}
   */
  public function answeringForm(array &$form, Answer $answer) {}

  /**
   * {@inheritdoc}
   */
  public function evaluationDisplay(Answer $answer): array {
    $data = $answer->getData();
    return [
      '#type' => 'fieldset',
      '#title' => $this->t('Student answer'),
      'answer' => [
        '#markup' => $data['answer'] ?? '',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function install(ActivityType $activity_type) {
    $this->pluginConfigInstaller->install($this->getPluginDefinition(), $activity_type->id());
  }

}
