<?php

namespace Drupal\lms\Plugin\ActivityAnswer;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\lms\Entity\ActivityType;
use Drupal\lms\Entity\Answer;

/**
 * Interface for Activity-answer plugins.
 */
interface ActivityAnswerInterface extends PluginInspectionInterface {

  /**
   * Get plugin id.
   */
  public function getId();

  /**
   * Indicates if answer should me evaluated on save or not.
   */
  public function evaluatedOnSave(Answer $activity);

  /**
   * Score logic for specified activity.
   */
  public function getScore(Answer $answer);

  /**
   * Modify answering form.
   */
  public function answeringForm(array &$form, Answer $answer);

  /**
   * Display user answer for teacher evaluation.
   *
   * Used only for activities that can be evaluated.
   *
   * @return array
   *   Renderable array.
   */
  public function evaluationDisplay(Answer $answer): array;

  /**
   * Perform install tasks.
   *
   * Executed after activity type config entity has been saved. Use to
   * install necessary field, form and display config.
   */
  public function install(ActivityType $activity_type);

}
