<?php

namespace Drupal\lms\Plugin\views\field;

use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\lms\Entity\CourseStatus as CourseStatusEntity;
use Drupal\views\Attribute\ViewsField;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * LMS Course user status field.
 */
#[ViewsField(id: 'lms_student_course_status')]
final class CourseStatus extends FieldPluginBase {

  /**
   * The constructor.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    private readonly AccountInterface $currentUser,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $status = $this->getValue($values);
    if ($status === NULL) {
      return $this->t('Not started');
    }

    $status_text = CourseStatusEntity::getStatusText($status);
    $status_class = \strtr($status, ' ', '-');

    $course_status = $values->_relationship_entities['course_status'] ?? NULL;
    if ($course_status === NULL) {
      $course_status = $values->_entity;
    }
    if (!$course_status instanceof CourseStatusEntity) {
      return [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#attributes' => [
          'class' => [$status_class],
        ],
        'text' => [
          '#markup' => $status_text,
        ],
      ];
    }

    // Output as link to results.
    return Link::createFromRoute($status_text, 'lms.group.results', [
      'group' => $course_status->getCourseId(),
      'user' => $course_status->getUserId(),
    ], [
      'attributes' => [
        '_target' => 'blank',
        'class' => [$status_class],
      ],
    ])->toRenderable();
  }

}
